@@ -12,7 +12,6 @@ import subprocess
 
 # Import third-party libs
 import yaml
-import zmq
 
 # Import salt libs
 from salt.exceptions import MinionError, SaltReqTimeoutError
@@ -21,6 +20,7 @@ import salt.crypt
 import salt.loader
 import salt.utils
 import salt.payload
+import salt.utils
 import salt.utils.templates
 from salt._compat import (
     URLError, HTTPError, BaseHTTPServer, urlparse, url_open)
@@ -69,7 +69,7 @@ class Client(object):
 
         filelist = []
 
-        for root, dirs, files in os.walk(destdir):
+        for root, dirs, files in os.walk(destdir, followlinks=True):
             for name in files:
                 path = os.path.join(root, name)
                 filelist.append(path)
@@ -96,7 +96,8 @@ class Client(object):
 
     def get_file(self, path, dest='', makedirs=False, env='base'):
         '''
-        Copies a file from the local files or master depending on implementation
+        Copies a file from the local files or master depending on
+        implementation
         '''
         raise NotImplementedError
 
@@ -138,6 +139,11 @@ class Client(object):
         '''
         ret = []
         path = self._check_proto(path)
+        log.info(
+            'Caching directory \'{0}\' for environment \'{1}\''.format(
+                path, env
+            )
+        )
         for fn_ in self.file_list(env):
             if fn_.startswith(path):
                 local = self.cache_file('salt://{0}'.format(fn_), env)
@@ -148,20 +154,20 @@ class Client(object):
         if include_empty:
             # Break up the path into a list containing the bottom-level directory
             # (the one being recursively copied) and the directories preceding it
-            separated = string.rsplit(path,'/',1)
-            if len(separated) != 2:
-                # No slashes in path. (This means all files in env will be copied)
-                prefix = ''
-            else:
-                prefix = separated[0]
+            #separated = string.rsplit(path, '/', 1)
+            #if len(separated) != 2:
+            #    # No slashes in path. (This means all files in env will be copied)
+            #    prefix = ''
+            #else:
+            #    prefix = separated[0]
             for fn_ in self.file_list_emptydirs(env):
                 if fn_.startswith(path):
-                    dest = os.path.normpath(
-                      os.sep.join([
-                      self.opts['cachedir'],
-                      'files',
-                      env])) 
-                    minion_dir = '%s/%s' % (dest,fn_)
+                    dest = salt.utils.path_join(
+                        self.opts['cachedir'],
+                        'files',
+                        env
+                    )
+                    minion_dir = '{0}/{1}'.format(dest, fn_)
                     if not os.path.isdir(minion_dir):
                         os.makedirs(minion_dir)
                     ret.append(minion_dir)
@@ -197,6 +203,12 @@ class Client(object):
         '''
         return []
 
+    def dir_list(self, env='base'):
+        '''
+        This function must be overwritten
+        '''
+        return []
+
     def is_cached(self, path, env='base'):
         '''
         Returns the full path to a file if it is cached locally on the minion
@@ -224,9 +236,9 @@ class Client(object):
             if path.endswith('.sls'):
                 # is an sls module!
                 if path.endswith('{0}init.sls'.format(os.sep)):
-                    states.append(path.replace(os.sep, '.')[:-9])
+                    states.append(path.replace('/', '.')[:-9])
                 else:
-                    states.append(path.replace(os.sep, '.')[:-4])
+                    states.append(path.replace('/', '.')[:-4])
         return states
 
     def get_state(self, sls, env):
@@ -266,16 +278,20 @@ class Client(object):
                 # Remove the leading directories from path to derive
                 # the relative path on the minion.
                 minion_relpath = string.lstrip(fn_[len(prefix):], '/')
-                ret.append(self.get_file('salt://{0}'.format(fn_),
-                                         '%s/%s' % (dest, minion_relpath),
-                                         True, env))
+                ret.append(
+                    self.get_file(
+                        'salt://{0}'.format(fn_),
+                        '{0}/{1}'.format(dest, minion_relpath),
+                        True, env
+                    )
+                )
         # Replicate empty dirs from master
         for fn_ in self.file_list_emptydirs(env):
             if fn_.startswith(path):
                 # Remove the leading directories from path to derive
                 # the relative path on the minion.
                 minion_relpath = string.lstrip(fn_[len(prefix):], '/')
-                minion_mkdir = '%s/%s' % (dest, minion_relpath)
+                minion_mkdir = '{0}/{1}'.format(dest, minion_relpath)
                 os.makedirs(minion_mkdir)
                 ret.append(minion_mkdir)
         ret.sort()
@@ -296,13 +312,13 @@ class Client(object):
                 else:
                     return ''
         else:
-            dest = os.path.normpath(
-                os.sep.join([
-                    self.opts['cachedir'],
-                    'extrn_files',
-                    env,
-                    url_data.netloc,
-                    url_data.path]))
+            dest = salt.utils.path_join(
+                self.opts['cachedir'],
+                'extrn_files',
+                env,
+                url_data.netloc,
+                url_data.path
+            )
             destdir = os.path.dirname(dest)
             if not os.path.isdir(destdir):
                 os.makedirs(destdir)
@@ -354,13 +370,13 @@ class Client(object):
             return ''
         if not dest:
             # No destination passed, set the dest as an extrn_files cache
-            dest = os.path.normpath(
-                os.sep.join([
-                    self.opts['cachedir'],
-                    'extrn_files',
-                    env,
-                    url_data.netloc,
-                    url_data.path]))
+            dest = salt.utils.path_join(
+                self.opts['cachedir'],
+                'extrn_files',
+                env,
+                url_data.netloc,
+                url_data.path
+            )
         destdir = os.path.dirname(dest)
         if not os.path.isdir(destdir):
             if makedirs:
@@ -413,7 +429,7 @@ class LocalClient(Client):
         if env not in self.opts['file_roots']:
             return ret
         for path in self.opts['file_roots'][env]:
-            for root, dirs, files in os.walk(path):
+            for root, dirs, files in os.walk(path, followlinks=True):
                 for fn in files:
                     ret.append(
                         os.path.relpath(
@@ -434,11 +450,23 @@ class LocalClient(Client):
         if env not in self.opts['file_roots']:
             return ret
         for path in self.opts['file_roots'][env]:
-            for root, dirs, files in os.walk(path):
+            for root, dirs, files in os.walk(path, followlinks=True):
                 if len(dirs) == 0 and len(files) == 0:
                     ret.append(os.path.relpath(root, path))
         return ret
 
+    def dir_list(self, env='base'):
+        '''
+        List the dirs in the file_roots
+        '''
+        ret = []
+        if env not in self.opts['file_roots']:
+            return ret
+        for path in self.opts['file_roots'][env]:
+            for root, dirs, files in os.walk(path, followlinks=True):
+                ret.append(os.path.relpath(root, path))
+        return ret
+
     def hash_file(self, path, env='base'):
         '''
         Return the hash of a file, to get the hash of a file in the file_roots
@@ -450,9 +478,8 @@ class LocalClient(Client):
             path = self._check_proto(path)
         except MinionError:
             if not os.path.isfile(path):
-                err = ('Specified file {0} is not present to generate '
-                        'hash').format(path)
-                log.warning(err)
+                err = 'Specified file {0} is not present to generate hash'
+                log.warning(err.format(path))
                 return ret
             else:
                 with open(path, 'rb') as f:
@@ -532,6 +559,7 @@ class RemoteClient(Client):
         dest is ommited, then the downloaded file will be placed in the minion
         cache
         '''
+        log.info('Fetching file \'{0}\''.format(path))
         path = self._check_proto(path)
         load = {'path': path,
                 'env': env,
@@ -612,6 +640,23 @@ class RemoteClient(Client):
         except SaltReqTimeoutError:
             return ''
 
+    def dir_list(self, env='base'):
+        '''
+        List the dirs on the master
+        '''
+        load = {'env': env,
+                'cmd': '_dir_list'}
+        try:
+            return self.auth.crypticle.loads(
+                    self.sreq.send(
+                        'aes',
+                        self.auth.crypticle.dumps(load),
+                        3,
+                        60)
+                    )
+        except SaltReqTimeoutError:
+            return ''
+
     def hash_file(self, path, env='base'):
         '''
         Return the hash of a file, to get the hash of a file on the salt
@@ -622,9 +667,8 @@ class RemoteClient(Client):
             path = self._check_proto(path)
         except MinionError:
             if not os.path.isfile(path):
-                err = ('Specified file {0} is not present to generate '
-                        'hash').format(path)
-                log.warning(err)
+                err = 'Specified file {0} is not present to generate hash'
+                log.warning(err.format(path))
                 return {}
             else:
                 ret = {}
