@@ -33,7 +33,8 @@ def usage(args=None):
         cmd = 'df -kP'
     else:
         cmd = 'df'
-    cmd = cmd + ' -' + args
+    if args:
+        cmd = cmd + ' -' + args
     ret = {}
     out = __salt__['cmd.run'](cmd).split('\n')
     for line in out:
@@ -64,7 +65,8 @@ def inodeusage(args=None):
         salt '*' disk.inodeusage
     '''
     cmd = 'df -i'
-    cmd = cmd + ' -' + args
+    if args is not None:
+        cmd = cmd + ' -' + args
     ret = {}
     out = __salt__['cmd.run'](cmd).split('\n')
     for line in out:
