@@ -1,8 +1,10 @@
 '''
 Manage groups on Linux
 '''
-
-import grp
+try:
+    import grp
+except ImportError:
+    pass
 
 
 def __virtual__():
