@@ -1,9 +1,11 @@
 '''
 Module for gathering and managing network information
 '''
-
+# Import Python libs
 import sys
-from string import ascii_letters, digits
+import logging
+
+# Import Salt libs
 from salt.utils.interfaces import *
 from salt.utils.socket_util import *
 
@@ -13,11 +15,13 @@ __outputter__ = {
     'netstat': 'txt',
 }
 
+log = logging.getLogger(__name__)
+
+
 def __virtual__():
     '''
     Only work on posix-like systems
     '''
-
     # Disable on Windows, a specific file module exists:
     if __grains__['os'] in ('Windows',):
         return False
@@ -25,14 +29,6 @@ def __virtual__():
     return 'network'
 
 
-def _sanitize_host(host):
-    '''
-    Sanitize host string.
-    '''
-    return "".join([
-        c for c in host[0:255] if c in (ascii_letters + digits + '.-')
-    ])
-
 def _cidr_to_ipv4_netmask(cidr_bits):
     '''
     Returns an IPv4 netmask
@@ -45,7 +41,7 @@ def _cidr_to_ipv4_netmask(cidr_bits):
             netmask += '255'
             cidr_bits -= 8
         else:
-            netmask += '%d' % (256-(2**(8-cidr_bits)))
+            netmask += '{0:d}'.format(256-(2**(8-cidr_bits)))
             cidr_bits = 0
     return netmask
 
@@ -58,6 +54,7 @@ def _number_of_set_bits_to_ipv4_netmask(set_bits):
     '''
     return _cidr_to_ipv4_netmask(_number_of_set_bits(set_bits))
 
+
 def _number_of_set_bits(x):
     '''
     Returns the number of bits that are set in a 32bit int
@@ -80,10 +77,10 @@ def _interfaces_ip(out):
     ret = dict()
 
     def parse_network(value, cols):
-        """
+        '''
         Return a tuple of ip, netmask, broadcast
         based on the current set of cols
-        """
+        '''
         brd = None
         if '/' in value:  # we have a CIDR in this address
             ip, cidr = value.split('/')
@@ -109,7 +106,7 @@ def _interfaces_ip(out):
                 continue
             m = re.match('^\d*:\s+([\w.]+)(?:@)?(\w+)?:\s+<(.+)>', line)
             if m:
-                iface,parent,attrs = m.groups()
+                iface, parent, attrs = m.groups()
                 if 'UP' in attrs.split(','):
                     data['up'] = True
                 else:
@@ -120,7 +117,7 @@ def _interfaces_ip(out):
 
             cols = line.split()
             if len(cols) >= 2:
-                type,value = tuple(cols[0:2])
+                type, value = tuple(cols[0:2])
                 if type in ('inet', 'inet6'):
                     if 'secondary' not in cols:
                         ipaddr, netmask, broadcast = parse_network(value, cols)
@@ -225,6 +222,13 @@ def _interfaces_ifconfig(out):
 
 
 def interfaces():
+    '''
+    Return a dictionary of information about all the interfaces on the minion
+
+    CLI Example::
+
+        salt '*' network.interfaces
+    '''
     ifaces = dict()
     if __salt__['cmd.has_exec']('ip'):
         cmd = __salt__['cmd.run']('ip addr show')
@@ -235,6 +239,82 @@ def interfaces():
     return ifaces
 
 
+def _get_net_start(ipaddr, netmask):
+    ipaddr_octets = ipaddr.split('.')
+    netmask_octets = netmask.split('.')
+    net_start_octets = [str(int(ipaddr_octets[x]) & int(netmask_octets[x]))
+                       for x in range(0, 4)]
+    return '.'.join(net_start_octets)
+
+
+def _get_net_size(mask):
+    binary_str = ''
+    for octet in mask.split('.'):
+        binary_str += bin(int(octet))[2:].zfill(8)
+    return len(binary_str.rstrip('0'))
+
+
+def _calculate_subnet(ipaddr, netmask):
+    return '{0}/{1}'.format(_get_net_start(ipaddr, netmask),
+                            _get_net_size(netmask))
+
+
+def _ipv4_to_bits(ipaddr):
+    '''
+    Accepts an IPv4 dotted quad and returns a string representing its binary
+    counterpart
+    '''
+    return ''.join([bin(int(x))[2:].rjust(8, '0') for x in ipaddr.split('.')])
+
+
+def subnets():
+    '''
+    Returns a list of subnets to which the host belongs
+    '''
+    ifaces = interfaces()
+    subnets = []
+
+    for ipv4_info in ifaces.values():
+        for ipv4 in ipv4_info.get('inet', []):
+            if ipv4['address'] == '127.0.0.1': continue
+            network = _calculate_subnet(ipv4['address'], ipv4['netmask'])
+            subnets.append(network)
+    return subnets
+
+
+def in_subnet(cidr):
+    '''
+    Returns True if host is within specified subnet, otherwise False
+    '''
+    try:
+        netstart, netsize = cidr.split('/')
+        netsize = int(netsize)
+    except:
+        log.error('Invalid CIDR \'{0}\''.format(cidr))
+        return False
+
+    ifaces = interfaces()
+
+    netstart_bin = _ipv4_to_bits(netstart)
+
+    if netsize < 32 and len(netstart_bin.rstrip('0')) > netsize:
+        log.error('Invalid network starting IP \'{0}\' in CIDR '
+                  '\'{1}\''.format(netstart, cidr))
+        return False
+
+    netstart_leftbits = netstart_bin[0:netsize]
+    for ipv4_info in ifaces.values():
+        for ipv4 in ipv4_info.get('inet', []):
+            if ipv4['address'] == '127.0.0.1': continue
+            if netsize == 32:
+                if netstart == ipv4['address']: return True
+            else:
+                ip_leftbits = _ipv4_to_bits(ipv4['address'])[0:netsize]
+                if netstart_leftbits == ip_leftbits: return True
+
+    return False
+
+
 def ping(host):
     '''
     Performs a ping to a host
@@ -243,7 +323,7 @@ def ping(host):
 
         salt '*' network.ping archlinux.org
     '''
-    cmd = 'ping -c 4 %s' % _sanitize_host(host)
+    cmd = 'ping -c 4 {0}'.format(_sanitize_host(host))
     return __salt__['cmd.run'](cmd)
 
 
@@ -296,7 +376,7 @@ def traceroute(host):
         salt '*' network.traceroute archlinux.org
     '''
     ret = []
-    cmd = 'traceroute %s' % _sanitize_host(host)
+    cmd = 'traceroute {0}'.format(_sanitize_host(host))
     out = __salt__['cmd.run'](cmd)
 
     for line in out:
@@ -327,7 +407,5 @@ def dig(host):
 
         salt '*' network.dig archlinux.org
     '''
-    cmd = 'dig %s' % _sanitize_host(host)
+    cmd = 'dig {0}'.format(_sanitize_host(host))
     return __salt__['cmd.run'](cmd)
-
-
