@@ -3,6 +3,8 @@ Service support for classic Red Hat type systems. This interface uses the
 service command (so it is compatible with upstart systems) and the chkconfig
 command.
 '''
+# Import Salt libs
+import salt.utils
 
 
 def __virtual__():
@@ -14,6 +16,8 @@ def __virtual__():
                'RedHat',
                'CentOS',
                'Scientific',
+               'CloudLinux',
+               'Amazon',
                'Fedora',
               ]
     if __grains__['os'] in enable:
@@ -120,6 +124,8 @@ def restart(name):
 
         salt '*' service.restart <service name>
     '''
+    if name == 'salt-minion':
+        salt.utils.daemonize_if(__opts__)
     cmd = '/sbin/service {0} restart'.format(name)
     return not __salt__['cmd.retcode'](cmd)
 
@@ -133,6 +139,8 @@ def status(name, sig=None):
 
         salt '*' service.status <service name>
     '''
+    if sig:
+        return bool(__salt__['status.pid'](sig))
     cmd = '/sbin/service {0} status'.format(name)
     return not __salt__['cmd.retcode'](cmd)
 
