@@ -1,9 +1,13 @@
 '''
 Provide the service module for systemd
 '''
+# Import Python libs
+import re
+# Import Salt libs
+import salt.utils
 
-import os
-
+VALID_UNIT_TYPES = ['service','socket', 'device', 'mount', 'automount',
+                    'swap', 'target', 'path', 'timer']
 
 def __virtual__():
     '''
@@ -16,6 +20,35 @@ def __virtual__():
     return False
 
 
+def _systemctl_cmd(action, name):
+    '''
+    Build a systemctl command line. Treat unit names without one
+    of the valid suffixes as a service.
+    '''
+    if not any(name.endswith(suffix) for suffix in VALID_UNIT_TYPES):
+        name += '.service'
+    return 'systemctl {0} {1}'.format(action, name)
+
+def _get_all_unit_files():
+    '''
+    Get all unit files and their state. Unit files ending in .service
+    are normalized so that they can be referenced without a type suffix.
+    '''
+    rexp = re.compile('(?m)^(?P<name>.+)\.(?P<type>' +
+                      '|'.join(VALID_UNIT_TYPES) +
+                      ')\s+(?P<state>.+)$')
+
+    out = __salt__['cmd.run_stdout']('systemctl --no-legend list-unit-files | col -b')
+
+    ret = {}
+    for match in rexp.finditer(out):
+        name = match.group('name')
+        if match.group('type') != 'service':
+            name += '.' + match.group('type')
+        ret[name] = match.group('state')
+    return ret
+
+
 def get_enabled():
     '''
     Return a list of all enabled services
@@ -25,10 +58,9 @@ def get_enabled():
         salt '*' service.get_enabled
     '''
     ret = []
-    for serv in get_all():
-        cmd = 'systemctl is-enabled {0}.service'.format(serv)
-        if not __salt__['cmd.retcode'](cmd):
-            ret.append(serv)
+    for name, state in _get_all_unit_files().iteritems():
+        if state == 'enabled':
+            ret.append(name)
     return sorted(ret)
 
 
@@ -41,10 +73,9 @@ def get_disabled():
         salt '*' service.get_disabled
     '''
     ret = []
-    for serv in get_all():
-        cmd = 'systemctl is-enabled {0}.service'.format(serv)
-        if __salt__['cmd.retcode'](cmd):
-            ret.append(serv)
+    for name, state in _get_all_unit_files().iteritems():
+        if state == 'disabled':
+            ret.append(name)
     return sorted(ret)
 
 
@@ -56,14 +87,7 @@ def get_all():
 
         salt '*' service.get_all
     '''
-    ret = set()
-    sdir = '/lib/systemd/system'
-    if not os.path.isdir('/lib/systemd/system'):
-        return []
-    for fn_ in os.listdir(sdir):
-        if fn_.endswith('.service'):
-            ret.add(fn_[:fn_.rindex('.')])
-    return sorted(list(ret))
+    return sorted(_get_all_unit_files().keys())
 
 
 def start(name):
@@ -74,8 +98,7 @@ def start(name):
 
         salt '*' service.start <service name>
     '''
-    cmd = 'systemctl start {0}.service'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    return not __salt__['cmd.retcode'](_systemctl_cmd('start', name))
 
 
 def stop(name):
@@ -86,8 +109,7 @@ def stop(name):
 
         salt '*' service.stop <service name>
     '''
-    cmd = 'systemctl stop {0}.service'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    return not __salt__['cmd.retcode'](_systemctl_cmd('stop', name))
 
 
 def restart(name):
@@ -98,8 +120,9 @@ def restart(name):
 
         salt '*' service.restart <service name>
     '''
-    cmd = 'systemctl restart {0}.service'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    if name == 'salt-minion':
+        salt.utils.daemonize_if(__opts__)
+    return not __salt__['cmd.retcode'](_systemctl_cmd('restart', name))
 
 
 def reload(name):
@@ -110,8 +133,7 @@ def reload(name):
 
         salt '*' service.reload <service name>
     '''
-    cmd = 'systemctl reload {0}.service'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    return not __salt__['cmd.retcode'](_systemctl_cmd('reload', name))
 
 
 # The unused sig argument is required to maintain consistency in the state
@@ -125,8 +147,7 @@ def status(name, sig=None):
 
         salt '*' service.status <service name>
     '''
-    cmd = 'systemctl show {0}.service'.format(name)
-    ret = __salt__['cmd.run'](cmd)
+    ret = __salt__['cmd.run'](_systemctl_cmd('show', name))
     index1 = ret.find('\nMainPID=')
     index2 = ret.find('\n', index1+9)
     mainpid = ret[index1+9:index2]
@@ -143,8 +164,7 @@ def enable(name):
 
         salt '*' service.enable <service name>
     '''
-    cmd = 'systemctl enable {0}.service'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    return not __salt__['cmd.retcode'](_systemctl_cmd('enable', name))
 
 
 def disable(name):
@@ -155,8 +175,7 @@ def disable(name):
 
         salt '*' service.disable <service name>
     '''
-    cmd = 'systemctl disable {0}.service'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    return not __salt__['cmd.retcode'](_systemctl_cmd('disable', name))
 
 
 def enabled(name):
@@ -167,8 +186,7 @@ def enabled(name):
 
         salt '*' service.enabled <service name>
     '''
-    cmd = 'systemctl is-enabled {0}.service'.format(name)
-    return not __salt__['cmd.retcode'](cmd)
+    return not __salt__['cmd.retcode'](_systemctl_cmd('is-enabled', name))
 
 
 def disabled(name):
@@ -179,5 +197,4 @@ def disabled(name):
 
         salt '*' service.disabled <service name>
     '''
-    cmd = 'systemctl is-enabled {0}.service'.format(name)
-    return bool(__salt__['cmd.retcode'](cmd))
+    return bool(__salt__['cmd.retcode'](_systemctl_cmd('is-enabled', name)))
