@@ -7,7 +7,6 @@ import os
 import copy
 import collections
 import logging
-import subprocess
 
 # Import Salt libs
 import salt.loader
@@ -17,10 +16,6 @@ import salt.crypt
 from salt._compat import string_types
 from salt.template import compile_template
 
-# Import third party libs
-import zmq
-import yaml
-
 log = logging.getLogger(__name__)
 
 
@@ -242,12 +237,6 @@ class Pillar(object):
                     for item in data:
                         if isinstance(item, string_types):
                             matches[env].append(item)
-        ext_matches = self.client.ext_nodes()
-        for env in ext_matches:
-            if env in matches:
-                matches[env] = list(set(ext_matches[env]).union(matches[env]))
-            else:
-                matches[env] = ext_matches[env]
         return matches
 
     def render_pstate(self, sls, env, mods):
@@ -337,7 +326,7 @@ class Pillar(object):
                         ext.update(self.ext_pillars[key](*val))
                     else:
                         ext.update(self.ext_pillars[key](val))
-                except Exception as e:
+                except Exception:
                     log.critical('Failed to load ext_pillar {0}'.format(key))
         return ext
 
