@@ -2,6 +2,18 @@
 Return data to a mongodb server
 
 Required python modules: pymongo
+
+
+This returner will send data from the minions to a MongoDB server. To 
+configure the settings for your MongoDB server, add the following lines
+to the minion config files::
+
+    mongo.db: <database name>
+    mongo.host: <server ip address>
+    mongo.user: <MongoDB username>
+    mongo.password: <MongoDB user password>
+    mongo.port: 27017
+
 '''
 
 import logging
@@ -15,6 +27,7 @@ except ImportError:
 
 log = logging.getLogger(__name__)
 
+# These options should be overridden in the minion config file
 __opts__ = {'mongo.db': 'salt',
             'mongo.host': 'salt',
             'mongo.password': '',
