@@ -66,7 +66,7 @@ takes a few arguments:
 Recursive directory management can also be set via the ``recurse``
 function. Recursive directory management allows for a directory on the salt
 master to be recursively copied down to the minion. This is a great tool for
-deploying large code and configuration systems. A recuse state would look
+deploying large code and configuration systems. A recurse state would look
 something like this:
 
 .. code-block:: yaml
@@ -78,6 +78,7 @@ something like this:
 # Import Python libs
 from contextlib import nested  # For < 2.7 compat
 import os
+import errno
 import shutil
 import difflib
 import hashlib
@@ -141,23 +142,56 @@ def _makedirs(path, user=None, group=None, mode=None):
     directory = os.path.dirname(path)
 
     if not os.path.isdir(directory):
-        os.makedirs(directory)
+        # turn on the executable bits for user, group and others.
+        # Note: the special bits are set to 0.
+        if mode:
+            mode = int(mode[-3:], 8) | 0111
+
+        _makedirs_perms(directory, user, group, mode)
         # If a caller such as managed() is invoked  with
         # makedirs=True, make sure that any created dirs
         # are created with the same user  and  group  to
         # follow the principal of least surprise method.
-        nmode = ''
-        if mode:
-            for char in mode:
-                if char == '0':
-                    nmode += char
-                elif int(char) % 2 == 1:
-                    # Is executable, continue
-                    nmode += char
-                else:
-                    # The mode is even, it need an executable bit
-                    nmode += str(int(char) + 1)
-        _check_perms(directory, None, user, group, nmode)
+
+
+
+def _makedirs_perms(name, user=None, group=None, mode=0755):
+    '''
+    Taken and modified from os.makedirs to set user, group and mode for each
+    directory created.
+    '''
+    path = os.path
+    mkdir = os.mkdir
+    head, tail = path.split(name)
+    if not tail:
+        head, tail = path.split(head)
+    if head and tail and not path.exists(head):
+        try:
+            _makedirs_perms(head, user, group, mode)
+        except OSError, e:
+            # be happy if someone already created the path
+            if e.errno != errno.EEXIST:
+                raise
+        if tail == os.curdir:  # xxx/newdir/. exists if xxx/newdir exists
+            return
+    mkdir(name)
+    _check_perms(name, None, user, group, int("%o" % mode) if mode else None)
+
+
+def _check_user(user, group):
+    '''
+    Checks if the named user and group are present on the minion
+    '''
+    err = ''
+    if user:
+        uid = __salt__['file.user_to_uid'](user)
+        if uid == '':
+            err += 'User {0} is not available '.format(user)
+    if group:
+        gid = __salt__['file.group_to_gid'](group)
+        if gid == '':
+            err += 'Group {0} is not available'.format(group)
+    return err
 
 
 def _is_bin(path):
@@ -214,6 +248,8 @@ def _clean_dir(root, keep):
     real_keep.add(root)
     if isinstance(keep, list):
         for fn_ in keep:
+            if not os.path.isabs(fn_):
+                continue
             real_keep.add(fn_)
             while True:
                 fn_ = os.path.dirname(fn_)
@@ -250,6 +286,7 @@ def _source_list(source, source_hash, env):
     if isinstance(source, list):
         # get the master file list
         mfiles = __salt__['cp.list_master'](env)
+        mdirs = __salt__['cp.list_master_dirs'](env)
         for single in source:
             if isinstance(single, dict):
                 # check the proto, if it is http or ftp then download the file
@@ -273,7 +310,7 @@ def _source_list(source, source_hash, env):
                         source_hash = single_hash
                         break
             elif isinstance(single, string_types):
-                if single in mfiles:
+                if single[7:] in mfiles or single[7:] in mdirs:
                     source = single
                     break
     return source, source_hash
@@ -379,26 +416,40 @@ def _get_managed(
 def _check_perms(name, ret, user, group, mode):
     '''
     Check the permissions on files and chown if needed
+
+    Note: 'mode' here is expected to be either a string or an integer,
+          in which case it will be converted into a base-10 string.
+
+          What this means is that in your YAML salt file, you can specify
+          mode as an integer(eg, 644) or as a string(eg, '644'). But, to
+          specify mode 0777, for example, it must be specified as the string,
+          '0777' otherwise, 0777 will be parsed as an octal and you'd get 511
+          instead.
     '''
     if not ret:
         ret = {'name': name,
                'changes': {},
-               'comment': '',
-               'result': False}
+               'comment': [],
+               'result': True}
+        orig_comment = ''
+    else:
+        orig_comment = ret['comment']
+        ret['comment'] = []
+
     # Check permissions
     perms = {}
     perms['luser'] = __salt__['file.get_user'](name)
     perms['lgroup'] = __salt__['file.get_group'](name)
-    perms['lmode'] = __salt__['file.get_mode'](name).lstrip('0')
+    perms['lmode'] = str(__salt__['file.get_mode'](name)).lstrip('0')
 
     # Mode changes if needed
     if mode:
-        if mode != perms['lmode']:
+        if str(mode) != perms['lmode']:
             if not __opts__['test']:
                 __salt__['file.set_mode'](name, mode)
-            if mode != __salt__['file.get_mode'](name).lstrip('0'):
+            if str(mode) != __salt__['file.get_mode'](name).lstrip('0'):
                 ret['result'] = False
-                ret['comment'] += 'Failed to change mode to {0} '.format(mode)
+                ret['comment'].append('Failed to change mode to {0}'.format(mode))
             else:
                 ret['changes']['mode'] = mode
     # user/group changes if needed, then check if it worked
@@ -414,28 +465,64 @@ def _check_perms(name, ret, user, group, mode):
                 user = perms['luser']
             if group is None:
                 group = perms['lgroup']
-            __salt__['file.chown'](
-                    name,
-                    user,
-                    group
-                    )
+            try:
+                __salt__['file.chown'](
+                        name,
+                        user,
+                        group
+                        )
+            except OSError:
+                ret['result'] = False
+
     if user:
         if user != __salt__['file.get_user'](name):
             ret['result'] = False
-            ret['comment'] = 'Failed to change user to {0} '.format(user)
+            ret['comment'].append('Failed to change user to {0}'.format(user))
         elif 'cuser' in perms:
             ret['changes']['user'] = user
     if group:
         if group != __salt__['file.get_group'](name):
             ret['result'] = False
-            ret['comment'] += ('Failed to change group to {0} '
+            ret['comment'].append('Failed to change group to {0}'
                                .format(group))
         elif 'cgroup' in perms:
             ret['changes']['group'] = group
 
+    if isinstance(orig_comment, basestring):
+        if orig_comment:
+            ret['comment'].insert(0, orig_comment)
+        ret['comment'] = '; '.join(ret['comment'])
     return ret, perms
 
 
+def _get_recurse_dest(prefix, fn_, source, env):
+    '''
+    Return the destination path to copy the file path, fn_(as returned by
+    a call to __salt__['cp.cache_dir']), to.
+    '''
+    local_roots = []
+    if __opts__['file_client'] == 'local':
+        local_roots = __opts__['file_roots'][env]
+        local_roots.sort(key=lambda p: len(p), reverse=True)
+
+    srcpath = source[7:] # the path after "salt://"
+
+    # in solo mode(ie, file_client=='local'), fn_ is a path below
+    # a file root; in remote mode, fn_ is a path below the cache_dir.
+    for root in local_roots:
+        n = len(root)
+        # if root is the longest prefix path of fn_
+        if root == fn_[:n]:
+            cachedir = os.path.join(root, srcpath)
+            break
+    else:
+        cachedir = os.path.join(
+                        __opts__['cachedir'], 'files', env, srcpath)
+
+    return os.path.join(prefix, os.path.relpath(fn_, cachedir))
+
+
+
 def _check_recurse(
         name,
         source,
@@ -456,17 +543,7 @@ def _check_recurse(
     for fn_ in __salt__['cp.cache_dir'](source, env, include_empty):
         if not fn_.strip():
             continue
-        dest = os.path.join(name,
-                os.path.relpath(
-                    fn_,
-                    os.path.join(
-                        __opts__['cachedir'],
-                        'files',
-                        env,
-                        source[7:]
-                        )
-                    )
-                )
+        dest = _get_recurse_dest(name, fn_, source, env)
         dirname = os.path.dirname(dest)
         if not dirname in vdir:
             # verify the directory perms if they are set
@@ -526,9 +603,14 @@ def _check_directory(
         if not 'group' in recurse:
             group = None
         for root, dirs, files in os.walk(name):
-            for name in files:
-                path = os.path.join(root, name)
-                fchange = _check_dir_meta(path, user, group, None)
+            for fname in files:
+                fchange = {}
+                path = os.path.join(root, fname)
+                stats = __salt__['file.stats'](path, 'md5')
+                if not user is None and user != stats['user']:
+                    fchange['user'] = user
+                if not group is None and group != stats['group']:
+                    fchange['group'] = group
                 if fchange:
                     changes[path] = fchange
             for name in dirs:
@@ -536,16 +618,13 @@ def _check_directory(
                 fchange = _check_dir_meta(path, user, group, None)
                 if fchange:
                     changes[path] = fchange
-    else:
-        if not os.path.isdir(name):
-            changes[name] = 'new'
-            return None, 'A new directory is set to be made at {0}'.format(
-                    name)
+    if not os.path.isdir(name):
+        changes[name] = {'directory': 'new'}
     if changes:
         comment = 'The following files will be changed:\n'
         for fn_ in changes:
             for key, val in changes[fn_].items():
-                comment += '{0}: {1} - {2}'.format(fn_, key, val)
+                comment += '{0}: {1} - {2}\n'.format(fn_, key, val)
         return None, comment
     return True, 'The directory {0} is in the correct state'.format(name)
 
@@ -650,7 +729,7 @@ def _check_file_meta(
                     slines = src.readlines()
                     nlines = name_.readlines()
                 changes['diff'] = (
-                        ''.join(difflib.unified_diff(nlines, slines))
+                        ''.join(difflib.unified_diff(slines, nlines))
                         )
             else:
                 changes['sum'] = 'Checksum differs'
@@ -671,7 +750,7 @@ def _check_touch(name, atime, mtime):
     Check to see if a file needs to be updated or created
     '''
     if not os.path.exists(name):
-        return None, 'File {0} is set to be created'
+        return None, 'File {0} is set to be created'.format(name)
     stats = __salt__['file.stats'](name)
     if not atime is None:
         if str(atime) != str(stats['atime']):
@@ -821,6 +900,26 @@ def absent(name):
     return ret
 
 
+def exists(name):
+    '''
+    Verify that the named file or directory is present or exists. 
+    Ensures pre-requisites outside of salts per-vue have been previously
+    satisified (aka, keytabs, private keys, etc.) before deployment
+
+    name
+        Absolute path which must exist
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'result': True,
+           'comment': ''}
+    if not os.path.exists(name):
+      return _error(ret, ('Specified path {0} does not exist').format(name))
+
+    ret['comment'] = 'Path {0} exists'.format(name)
+    return ret
+
+
 def managed(name,
         source=None,
         source_hash='',
@@ -882,8 +981,9 @@ def managed(name,
         file.
 
     replace
-        If this file should be replaced, if false then this command will
-        be ignored if the file exists already. Default is true.
+        If this file should be replaced.  If false, this command will
+        not overwrite file contents but will enforce permissions if the file 
+        exists already.  Default is true.
 
     context
         Overrides default context variables passed to the template.
@@ -900,6 +1000,10 @@ def managed(name,
            'comment': '',
            'name': name,
            'result': True}
+    u_check = _check_user(user, group)
+    if u_check:
+        # The specified user or group do not exist
+        return _error(ret, u_check)
     if not os.path.isabs(name):
         return _error(
             ret, ('Specified file {0} is not an absolute'
@@ -914,7 +1018,13 @@ def managed(name,
 
     if not replace:
         if os.path.exists(name):
-            ret['comment'] = 'File {0} exists. No changes made'.format(name)
+           # Check and set the permissions if necessary
+            ret, perms = _check_perms(name, ret, user, group, mode)
+            if __opts__['test']:
+                ret['comment'] = 'File {0} not updated'.format(name)
+            elif not ret['changes'] and ret['result']:
+                ret['comment'] = ('File {0} exists with proper permissions.'
+                                  '  No changes made.').format(name)
             return ret
         if not source:
             return touch(name, makedirs=makedirs)
@@ -1151,6 +1261,10 @@ def directory(name,
            'changes': {},
            'result': True,
            'comment': ''}
+    u_check = _check_user(user, group)
+    if u_check:
+        # The specified user or group do not exist
+        return _error(ret, u_check)
     if not os.path.isabs(name):
         return _error(
             ret, 'Specified file {0} is not an absolute path'.format(name))
@@ -1338,7 +1452,12 @@ def recurse(name,
     ret = {'name': name,
            'changes': {},
            'result': True,
-           'comment': ''}
+           'comment': {}  # { path: [comment, ...] }
+           }
+    u_check = _check_user(user, group)
+    if u_check:
+        # The specified user or group do not exist
+        return _error(ret, u_check)
     if not os.path.isabs(name):
         return _error(
             ret, 'Specified file {0} is not an absolute path'.format(name))
@@ -1352,7 +1471,9 @@ def recurse(name,
             # it is not a dir, but it exists - fail out
             return _error(
                 ret, 'The path {0} exists and is not a directory'.format(name))
-        os.makedirs(name)
+        _makedirs_perms(name, user, group,
+                        int(str(dir_mode), 8) if dir_mode else None)
+
     if __opts__['test']:
         ret['result'], ret['comment'] = _check_recurse(
                 name,
@@ -1366,36 +1487,42 @@ def recurse(name,
                 env,
                 include_empty)
         return ret
+
+    def update_changes_by_perms(path, mode, changetype='updated'):
+        _ret = {'name': name,
+                'changes': {},
+                'result': True,
+                'comment': []
+               }
+        _check_perms(path, _ret, user, group, mode)
+        ret['result'] &= _ret['result'] # ie, once false, stay false.
+        if _ret['comment']:
+            comments = ret['comment'].setdefault(path, [])
+            comments.extend(_ret['comment'])
+        if _ret['changes']:
+            ret['changes'][path] = changetype
+
+    # If source is a list, find which in the list actually exists
+    source, source_hash = _source_list(source, '', env)
+
     vdir = set()
     for fn_ in __salt__['cp.cache_dir'](source, env, include_empty):
         if not fn_.strip():
             continue
-        dest = os.path.join(name,
-                os.path.relpath(
-                    fn_,
-                    os.path.join(
-                        __opts__['cachedir'],
-                        'files',
-                        env,
-                        source[7:]
-                        )
-                    )
-                )
+        # fn_ here is the absolute source path of the file to copy from;
+        # it is either a normal file or an empty dir(if include_empthy==true).
+
+        dest = _get_recurse_dest(name, fn_, source, env)
         dirname = os.path.dirname(dest)
+        keep.add(dest)
         if not os.path.isdir(dirname):
             _makedirs(dest, user=user, group=group)
         if not dirname in vdir:
             # verify the directory perms if they are set
-            # _check_perms(name, ret, user, group, mode)
-            _ret, perms = _check_perms(dirname, {}, user, group, dir_mode)
-            if _ret['changes']:
-                ret['changes'][dirname] = 'updated'
+            update_changes_by_perms(dirname, dir_mode)
             vdir.add(dirname)
         if os.path.isfile(dest):
-            _ret, perms = _check_perms(dest, {}, user, group, file_mode)
-            if _ret['changes']:
-                ret['changes'][dest] = 'updated'
-            keep.add(dest)
+            update_changes_by_perms(dest, file_mode)
             srch = ''
             dsth = ''
             # The file is present, if the sum differes replace it
@@ -1404,35 +1531,28 @@ def recurse(name,
                 dsth = hashlib.md5(dst_.read()).hexdigest()
             if srch != dsth:
                 # The downloaded file differes, replace!
-                # FIXME: no metadata (ownership, permissions) available
                 salt.utils.copyfile(
                         fn_,
                         dest,
                         _backup_mode(backup),
                         __opts__['cachedir'])
-                ret['changes'][dest] = 'updated'
+                update_changes_by_perms(dest, file_mode)
         elif os.path.isdir(dest) and include_empty:
             #check perms
-            _ret, perms = _check_perms(dest, {}, user, group, dir_mode)
-            if _ret['changes']:
-                ret['changes'][dest] = 'updated'
-            keep.add(dest)
+            update_changes_by_perms(dest, dir_mode)
         else:
-            keep.add(dest)
             if os.path.isdir(fn_) and include_empty:
                 #create empty dir
                 os.mkdir(dest)
-
-                if dir_mode:
-                    __salt__['file.set_mode'](dest, dir_mode)
+                update_changes_by_perms(dest, dir_mode)
             else:
                 # The destination file is not present, make it
-                # FIXME: no metadata (ownership, permissions) available
                 salt.utils.copyfile(
                         fn_,
                         dest,
                         _backup_mode(backup),
                         __opts__['cachedir'])
+                update_changes_by_perms(dest, file_mode)
             ret['changes'][dest] = 'new'
     keep = list(keep)
     if clean:
@@ -1501,15 +1621,23 @@ def sed(name, before, after, limit='', backup='.bak', options='-r -e',
         ret['comment'] = 'File {0} is set to be updated'.format(name)
         ret['result'] = None
         return ret
+    with open(name, 'rb') as fp_:
+        slines = fp_.readlines()
     # should be ok now; perform the edit
     __salt__['file.sed'](name, before, after, limit, backup, options, flags)
+    with open(name, 'rb') as fp_:
+        nlines = fp_.readlines()
 
     # check the result
     ret['result'] = __salt__['file.contains_regex'](name, after)
+    if slines != nlines:
+        # Changes happened, add them
+        ret['changes']['diff'] = (
+                ''.join(difflib.unified_diff(slines, nlines))
+                )
 
     if ret['result']:
         ret['comment'] = 'File successfully edited'
-        ret['changes'].update({'old': before, 'new': after})
     else:
         ret['comment'] = 'Expected edit does not appear in file'
 
@@ -1518,6 +1646,27 @@ def sed(name, before, after, limit='', backup='.bak', options='-r -e',
 
 def comment(name, regex, char='#', backup='.bak'):
     '''
+    Comment out specified lines in a file.
+
+    path
+        The full path to the file to be edited
+    regex
+        A regular expression used to find the lines that are to be commented;
+        this pattern will be wrapped in parenthesis and will move any
+        preceding/trailing ``^`` or ``$`` characters outside the parenthesis
+        (e.g., the pattern ``^foo$`` will be rewritten as ``^(foo)$``)
+    char : ``#``
+        The character to be inserted at the beginning of a line in order to
+        comment it out
+    backup : ``.bak``
+        The file will be backed up before edit with this file extension
+
+        .. warning::
+
+            This backup will be overwritten each time ``sed`` / ``comment`` /
+            ``uncomment`` is called. Meaning the backup will only be useful
+            after the first invocation.
+
     Usage::
 
         /etc/fstab:
@@ -1547,16 +1696,25 @@ def comment(name, regex, char='#', backup='.bak'):
         ret['comment'] = 'File {0} is set to be updated'.format(name)
         ret['result'] = None
         return ret
+    with open(name, 'rb') as fp_:
+        slines = fp_.readlines()
     # Perform the edit
     __salt__['file.comment'](name, regex, char, backup)
 
+    with open(name, 'rb') as fp_:
+        nlines = fp_.readlines()
+
     # Check the result
     ret['result'] = __salt__['file.contains_regex'](name, unanchor_regex)
 
+    if slines != nlines:
+        # Changes happened, add them
+        ret['changes']['diff'] = (
+                ''.join(difflib.unified_diff(slines, nlines))
+                )
+
     if ret['result']:
         ret['comment'] = 'Commented lines successfully'
-        ret['changes'] = {'old': '',
-                'new': 'Commented lines matching: {0}'.format(regex)}
     else:
         ret['comment'] = 'Expected commented lines not found'
 
@@ -1565,6 +1723,23 @@ def comment(name, regex, char='#', backup='.bak'):
 
 def uncomment(name, regex, char='#', backup='.bak'):
     '''
+    Uncomment specified commented lines in a file
+
+    path
+        The full path to the file to be edited
+    regex
+        A regular expression used to find the lines that are to be uncommented.
+        This regex should not include the comment character. A leading ``^``
+        character will be stripped for convenience (for easily switching
+        between comment() and uncomment()).
+    char : ``#``
+        The character to remove in order to uncomment a line; if a single
+        whitespace character follows the comment it will also be removed
+    backup : ``.bak``
+        The file will be backed up before edit with this file extension;
+        **WARNING:** each time ``sed``/``comment``/``uncomment`` is called will
+        overwrite this backup
+
     Usage::
 
         /etc/adduser.conf:
@@ -1579,8 +1754,6 @@ def uncomment(name, regex, char='#', backup='.bak'):
     if not check_res:
         return _error(ret, check_msg)
 
-    unanchor_regex = regex.lstrip('^')
-
     # Make sure the pattern appears in the file
     if __salt__['file.contains_regex'](name, regex):
         ret['comment'] = 'Pattern already uncommented'
@@ -1596,23 +1769,34 @@ def uncomment(name, regex, char='#', backup='.bak'):
         ret['comment'] = 'File {0} is set to be updated'.format(name)
         ret['result'] = None
         return ret
+
+    with open(name, 'rb') as fp_:
+        slines = fp_.readlines()
+
     # Perform the edit
     __salt__['file.uncomment'](name, regex, char, backup)
 
+    with open(name, 'rb') as fp_:
+        nlines = fp_.readlines()
+
     # Check the result
     ret['result'] = __salt__['file.contains_regex'](name, regex)
 
+    if slines != nlines:
+        # Changes happened, add them
+        ret['changes']['diff'] = (
+                ''.join(difflib.unified_diff(slines, nlines))
+                )
+
     if ret['result']:
         ret['comment'] = 'Uncommented lines successfully'
-        ret['changes'] = {'old': '',
-                'new': 'Uncommented lines matching: {0}'.format(regex)}
     else:
         ret['comment'] = 'Expected uncommented lines not found'
 
     return ret
 
 
-def append(name, text):
+def append(name, text=None, makedirs=False, source=None, source_hash=None):
     '''
     Ensure that some text appears at the end of a file
 
@@ -1640,35 +1824,81 @@ def append(name, text):
     '''
     ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}
 
+    if makedirs:
+        dirname = os.path.dirname(name)
+        if not __salt__['file.directory_exists'](dirname):
+            _makedirs(name)
+            check_res, check_msg = _check_directory(
+                dirname, None, None, False, None, False, False
+            )
+            if not check_res:
+                return _error(ret, check_msg)
+
+        # Make sure that we have a file
+        __salt__['file.touch'](name)
+
     check_res, check_msg = _check_file(name)
     if not check_res:
         return _error(ret, check_msg)
 
+    if source:
+        # get cached file or copy it to cache
+        cached_source_path = __salt__['cp.cache_file'](source)
+        logger.debug(
+            "state file.append cached source {0} -> {1}".format(
+                source, cached_source_path
+            )
+        )
+        cached_source = managed(
+            cached_source_path, source=source, source_hash=source_hash
+        )
+        if cached_source['result'] is True:
+            logger.debug(
+                "state file.append is loading text contents from cached source "
+                "{0}({1})".format(source, cached_source_path)
+            )
+            text = open(cached_source_path, 'r').read()
+
     if isinstance(text, string_types):
         text = (text,)
 
+    with open(name, 'rb') as fp_:
+        slines = fp_.readlines()
+
+    count = 0
+
     for chunk in text:
+
+        if __salt__['file.contains_regex'](
+                        name, salt.utils.build_whitepace_splited_regex(chunk)):
+            continue
+
         try:
             lines = chunk.split('\n')
         except AttributeError:
-            logger.debug('Error appending text to %s; given object is: %s',
-                    name, type(chunk))
+            logger.debug(
+                'Error appending text to {0}; given object is: {1}'.format(
+                    name, type(chunk)
+                )
+            )
             return _error(ret, 'Given text is not a string')
 
         for line in lines:
-            if __salt__['file.contains'](name, line):
-                continue
-            else:
-                if __opts__['test']:
-                    ret['comment'] = 'File {0} is set to be updated'.format(
-                            name)
-                    ret['result'] = None
-                    return ret
-                __salt__['file.append'](name, line)
-                cgs = ret['changes'].setdefault('new', [])
-                cgs.append(line)
+            if __opts__['test']:
+                ret['comment'] = 'File {0} is set to be updated'.format(name)
+                ret['result'] = None
+                return ret
+            __salt__['file.append'](name, line)
+            count += 1
 
-    count = len(ret['changes'].get('new', []))
+    with open(name, 'rb') as fp_:
+        nlines = fp_.readlines()
+
+    if slines != nlines:
+        # Changes happened, add them
+        ret['changes']['diff'] = (
+                ''.join(difflib.unified_diff(slines, nlines))
+                )
 
     ret['comment'] = 'Appended {0} lines'.format(count)
     ret['result'] = True
@@ -1693,7 +1923,8 @@ def touch(name, atime=None, mtime=None, makedirs=False):
     }
     if not os.path.isabs(name):
         return _error(
-            ret, 'Specified file {0} is not an absolute path'.format(name))
+            ret, 'Specified file {0} is not an absolute path'.format(name)
+        )
 
     if __opts__['test']:
         ret['result'], ret['comment'] = _check_touch(name, atime, mtime)
@@ -1703,7 +1934,8 @@ def touch(name, atime=None, mtime=None, makedirs=False):
         _makedirs(name)
     if not os.path.isdir(os.path.dirname(name)):
         return _error(
-            ret, 'Directory not present to touch file {0}'.format(name))
+            ret, 'Directory not present to touch file {0}'.format(name)
+        )
     exists = os.path.exists(name)
     ret['result'] = __salt__['file.touch'](name, atime, mtime)
 
@@ -1711,7 +1943,9 @@ def touch(name, atime=None, mtime=None, makedirs=False):
         ret['comment'] = 'Created empty file {0}'.format(name)
         ret['changes']['new'] = name
     elif exists and ret['result']:
-        ret['comment'] = 'Updated times on file {0}'.format(name)
+        ret['comment'] = 'Updated times on {0} {1}'.format(
+            'directory' if os.path.isdir(name) else 'file', name
+        )
 
     return ret
 
