@@ -5,6 +5,9 @@ Interaction with Git repositories.
 NOTE: This modules is under heavy development and the API is subject to change.
 It may be replaced with a generic VCS module if this proves viable.
 
+Important, before using git over ssh, make sure your remote host fingerprint
+exists in "~/.ssh/known_hosts" file.
+
 .. code-block:: yaml
 
     https://github.com/saltstack/salt.git:
@@ -71,13 +74,13 @@ def latest(name,
                     ('Repository {0} update is probably required (current '
                     'revision is {1})').format(target, current_rev))
 
-        __salt__['git.pull'](target, user=runas)
+        pull_out = __salt__['git.pull'](target, user=runas)
 
         if rev:
             __salt__['git.checkout'](target, rev, user=runas)
 
         if submodules:
-            __salt__['git.submodule'](target, user=runas)
+            __salt__['git.submodule'](target, user=runas, opts='--recursive')
 
         new_rev = __salt__['git.revision'](cwd=target, user=runas)
         if current_rev != new_rev:
@@ -87,6 +90,13 @@ def latest(name,
             ret['comment'] = 'Repository {0} updated'.format(target)
             ret['changes']['revision'] = '{0} => {1}'.format(
                     current_rev, new_rev)
+        else:
+            # Check that there was no error preventing the revision update
+            if 'error:' in pull_out:
+                return _fail(
+                    ret,
+                    'An error was thrown by git:\n{0}'.format(pull_out)
+                    )
     else:
         if os.path.isdir(target):
             # git clone is required, but target exists -- however it is empty
