@@ -20,25 +20,6 @@ def __virtual__():
     return 'service'
 
 
-def _get_stat(name, sig):
-    '''
-    Return the status of a service based on signature and status, if the
-    signature is used then the status option will be ignored
-    '''
-    stat = False
-    if sig:
-        if sig == 'detect':
-            cmd = "{0[ps]} | grep {1} | grep -v grep | awk '{{print $2}}'"
-            cmd = cmd.format(__grains__, name)
-        else:
-            cmd = "{0[ps]} | grep {1} | grep -v grep | awk '{{print $2}}'"
-            cmd = cmd.format(__grains__, sig)
-        stat = bool(__salt__['cmd.run'](cmd))
-    else:
-        stat = __salt__['service.status'](name)
-    return stat
-
-
 def _enable(name, started):
     '''
     Enable the service
@@ -67,7 +48,6 @@ def _enable(name, started):
     if __salt__['service.enabled'](name):
         # Service is enabled
         if started is True:
-            ret['changes'][name] = True
             ret['comment'] = ('Service {0} is already enabled,'
                 ' and is running').format(name)
             return ret
@@ -227,7 +207,7 @@ def running(name, enable=None, sig=None):
            'result': True,
            'comment': ''}
     # See if the service is already running
-    if _get_stat(name, sig):
+    if __salt__['service.status'](name, sig):
         ret['comment'] = 'The service {0} is already running'.format(name)
         if enable is True:
             return _enable(name, None)
@@ -293,7 +273,7 @@ def dead(name, enable=None, sig=None):
            'changes': {},
            'result': True,
            'comment': ''}
-    if not _get_stat(name, sig):
+    if not __salt__['service.status'](name, sig):
         ret['comment'] = 'The service {0} is already dead'.format(name)
         if enable is True:
             return _enable(name, None)
@@ -301,6 +281,17 @@ def dead(name, enable=None, sig=None):
             return _disable(name, None)
         else:
             return ret
+
+    # Check if the service is available
+    if 'service.get_all' in __salt__:
+        # get_all is available, we can reliable check for the service
+        services = __salt__['service.get_all']()
+        if not name in services:
+            ret['result'] = False
+            ret['comment'] = 'The named service {0} is not available'.format(
+                    name)
+            return ret
+
     if __opts__['test']:
         ret['result'] = None
         ret['comment'] = 'Service {0} is set to be killed'.format(name)
@@ -352,7 +343,7 @@ def disabled(name):
     return _disable(name, None)
 
 
-def mod_watch(name, sig=None, reload=False):
+def mod_watch(name, sig=None, reload=False, full_restart=False):
     '''
     The service watcher, called to invoke the watch command.
 
@@ -364,15 +355,18 @@ def mod_watch(name, sig=None, reload=False):
     '''
     if __salt__['service.status'](name, sig):
         if 'service.reload' in __salt__ and reload:
-            changes = {name: __salt__['service.reload'](name)}
+            restart_func = __salt__['service.reload']
+        elif 'service.full_restart' in __salt__ and full_restart:
+            restart_func = __salt__['service.full_restart']
         else:
-            changes = {name: __salt__['service.restart'](name)}
-        return {'name': name,
-                'changes': changes,
-                'result': True,
-                'comment': 'Service restarted'}
+            restart_func = __salt__['service.restart']
+    else:
+        restart_func = __salt__['service.start']
 
+    result = restart_func(name)
     return {'name': name,
-            'changes': {},
-            'result': True,
-            'comment': 'Service {0} started'.format(name)}
+            'changes': {name: result},
+            'result': result,
+            'comment': 'Service restarted' if result else \
+                       'Failed to restart the service'
+           }
