@@ -2,6 +2,7 @@
 Tests for the file state
 '''
 import os
+import shutil
 import integration
 
 
@@ -9,6 +10,7 @@ class FileTest(integration.ModuleCase):
     '''
     Validate the file state
     '''
+
     def test_symlink(self):
         '''
         file.symlink
@@ -16,7 +18,7 @@ class FileTest(integration.ModuleCase):
         name = os.path.join(integration.TMP, 'symlink')
         tgt = os.path.join(integration.TMP, 'target')
         ret = self.run_state('file.symlink', name=name, target=tgt)
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
 
     def test_test_symlink(self):
@@ -26,7 +28,7 @@ class FileTest(integration.ModuleCase):
         name = os.path.join(integration.TMP, 'symlink')
         tgt = os.path.join(integration.TMP, 'target')
         ret = self.run_state('file.symlink', test=True, name=name, target=tgt)
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
 
     def test_absent_file(self):
@@ -37,7 +39,7 @@ class FileTest(integration.ModuleCase):
         with open(name, 'w+') as fp_:
             fp_.write('killme')
         ret = self.run_state('file.absent', name=name)
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
         self.assertFalse(os.path.isfile(name))
 
@@ -46,9 +48,11 @@ class FileTest(integration.ModuleCase):
         file.absent
         '''
         name = os.path.join(integration.TMP, 'dir_to_kill')
-        os.makedirs(name)
+        if not os.path.isdir(name):
+            # left behind... Don't fail because of this!
+            os.makedirs(name)
         ret = self.run_state('file.absent', name=name)
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
         self.assertFalse(os.path.isdir(name))
 
@@ -57,11 +61,14 @@ class FileTest(integration.ModuleCase):
         file.absent
         '''
         name = os.path.join(integration.TMP, 'link_to_kill')
-        os.symlink(name, '{0}.tgt'.format(name))
+        if not os.path.islink('{0}.tgt'.format(name)):
+            os.symlink(name, '{0}.tgt'.format(name))
         ret = self.run_state('file.absent', name=name)
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
         self.assertFalse(os.path.islink(name))
+        if os.path.islink('{0}.tgt'.format(name)):
+            os.unlink('{0}.tgt'.format(name))
 
     def test_test_absent(self):
         '''
@@ -71,9 +78,10 @@ class FileTest(integration.ModuleCase):
         with open(name, 'w+') as fp_:
             fp_.write('killme')
         ret = self.run_state('file.absent', test=True, name=name)
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
         self.assertTrue(os.path.isfile(name))
+        os.remove(name)
 
     def test_managed(self):
         '''
@@ -81,22 +89,17 @@ class FileTest(integration.ModuleCase):
         '''
         name = os.path.join(integration.TMP, 'grail_scene33')
         ret = self.run_state(
-                'file.managed',
-                name=name,
-                source='salt://grail/scene33')
+            'file.managed', name=name, source='salt://grail/scene33'
+        )
         src = os.path.join(
-                integration.FILES,
-                'file',
-                'base',
-                'grail',
-                'scene33'
-                )
+            integration.FILES, 'file', 'base', 'grail', 'scene33'
+        )
         with open(src, 'r') as fp_:
             master_data = fp_.read()
         with open(name, 'r') as fp_:
             minion_data = fp_.read()
         self.assertEqual(master_data, minion_data)
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
 
     def test_test_managed(self):
@@ -105,12 +108,10 @@ class FileTest(integration.ModuleCase):
         '''
         name = os.path.join(integration.TMP, 'grail_not_scene33')
         ret = self.run_state(
-                'file.managed',
-                test=True,
-                name=name,
-                source='salt://grail/scene33')
+            'file.managed', test=True, name=name, source='salt://grail/scene33'
+        )
         self.assertFalse(os.path.isfile(name))
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
 
     def test_directory(self):
@@ -118,12 +119,9 @@ class FileTest(integration.ModuleCase):
         file.directory
         '''
         name = os.path.join(integration.TMP, 'a_new_dir')
-        ret = self.run_state(
-                'file.directory',
-                name=name,
-                )
+        ret = self.run_state('file.directory', name=name)
         self.assertTrue(os.path.isdir(name))
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
 
     def test_test_directory(self):
@@ -131,13 +129,9 @@ class FileTest(integration.ModuleCase):
         file.directory
         '''
         name = os.path.join(integration.TMP, 'a_not_dir')
-        ret = self.run_state(
-                'file.directory',
-                test=True,
-                name=name,
-                )
+        ret = self.run_state('file.directory', test=True, name=name)
         self.assertFalse(os.path.isdir(name))
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
 
     def test_recurse(self):
@@ -145,14 +139,11 @@ class FileTest(integration.ModuleCase):
         file.recurse
         '''
         name = os.path.join(integration.TMP, 'recurse_dir')
-        ret = self.run_state(
-                'file.recurse',
-                name=name,
-                source='salt://grail',
-                )
+        ret = self.run_state('file.recurse', name=name, source='salt://grail')
         self.assertTrue(os.path.isfile(os.path.join(name, '36', 'scene')))
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
+        shutil.rmtree(name, ignore_errors=True)
 
     def test_test_recurse(self):
         '''
@@ -160,14 +151,12 @@ class FileTest(integration.ModuleCase):
         '''
         name = os.path.join(integration.TMP, 'recurse_test_dir')
         ret = self.run_state(
-                'file.recurse',
-                test=True,
-                name=name,
-                source='salt://grail',
-                )
+            'file.recurse', test=True, name=name, source='salt://grail',
+        )
         self.assertFalse(os.path.isfile(os.path.join(name, '36', 'scene')))
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
+        os.removedirs(name)
 
     def test_sed(self):
         '''
@@ -177,15 +166,13 @@ class FileTest(integration.ModuleCase):
         with open(name, 'w+') as fp_:
             fp_.write('change_me')
         ret = self.run_state(
-                'file.sed',
-                name=name,
-                before='change',
-                after='salt'
-                )
+            'file.sed', name=name, before='change', after='salt'
+        )
         with open(name, 'r') as fp_:
             self.assertIn('salt', fp_.read())
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
+        os.remove(name)
 
     def test_test_sed(self):
         '''
@@ -195,16 +182,13 @@ class FileTest(integration.ModuleCase):
         with open(name, 'w+') as fp_:
             fp_.write('change_me')
         ret = self.run_state(
-                'file.sed',
-                test=True,
-                name=name,
-                before='change',
-                after='salt'
-                )
+            'file.sed', test=True, name=name, before='change', after='salt'
+        )
         with open(name, 'r') as fp_:
             self.assertIn('change', fp_.read())
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
+        os.remove(name)
 
     def test_comment(self):
         '''
@@ -230,6 +214,7 @@ class FileTest(integration.ModuleCase):
         # result is still positive
         ret = list(_ret.values())[0]
         self.assertTrue(ret['result'], ret)
+        os.remove(name)
 
     def test_test_comment(self):
         '''
@@ -239,15 +224,13 @@ class FileTest(integration.ModuleCase):
         with open(name, 'w+') as fp_:
             fp_.write('comment_me')
         ret = self.run_state(
-                'file.comment',
-                test=True,
-                name=name,
-                regex='.*comment.*',
-                )
+            'file.comment', test=True, name=name, regex='.*comment.*',
+        )
         with open(name, 'r') as fp_:
             self.assertNotIn('#comment', fp_.read())
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
+        os.remove(name)
 
     def test_uncomment(self):
         '''
@@ -259,8 +242,9 @@ class FileTest(integration.ModuleCase):
         ret = self.run_state('file.uncomment', name=name, regex='^comment')
         with open(name, 'r') as fp_:
             self.assertNotIn('#comment', fp_.read())
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
+        os.remove(name)
 
     def test_test_uncomment(self):
         '''
@@ -270,15 +254,13 @@ class FileTest(integration.ModuleCase):
         with open(name, 'w+') as fp_:
             fp_.write('#comment_me')
         ret = self.run_state(
-                'file.uncomment',
-                test=True,
-                name=name,
-                regex='^comment.*',
-                )
+            'file.uncomment', test=True, name=name, regex='^comment.*'
+        )
         with open(name, 'r') as fp_:
             self.assertIn('#comment', fp_.read())
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
+        os.remove(name)
 
     def test_append(self):
         '''
@@ -287,15 +269,12 @@ class FileTest(integration.ModuleCase):
         name = os.path.join(integration.TMP, 'append_test')
         with open(name, 'w+') as fp_:
             fp_.write('#salty!')
-        ret = self.run_state(
-                'file.append',
-                name=name,
-                text='cheese',
-                )
+        ret = self.run_state('file.append', name=name, text='cheese')
         with open(name, 'r') as fp_:
             self.assertIn('cheese', fp_.read())
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
+        os.remove(name)
 
     def test_test_append(self):
         '''
@@ -304,44 +283,91 @@ class FileTest(integration.ModuleCase):
         name = os.path.join(integration.TMP, 'append_test_test')
         with open(name, 'w+') as fp_:
             fp_.write('#salty!')
-        ret = self.run_state(
-                'file.append',
-                test=True,
-                name=name,
-                text='cheese',
-                )
+        ret = self.run_state('file.append', test=True, name=name, text='cheese')
         with open(name, 'r') as fp_:
             self.assertNotIn('cheese', fp_.read())
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
+        os.remove(name)
+
+    def test_append_issue_1864_makedirs(self):
+        '''
+        file.append but create directories if needed as an option
+        '''
+        fname = 'append_issue_1864_makedirs'
+        name = os.path.join(integration.TMP, fname)
+        ret = self.run_state('file.append', name=name, text='cheese')
+        result = self.state_result(ret)
+        self.assertFalse(result)
+
+        try:
+            # Non existing file get's touched
+            if os.path.isfile(name):
+                # left over
+                os.remove(name)
+            ret = self.run_state(
+                'file.append', name=name, text='cheese', makedirs=True
+            )
+            result = self.state_result(ret)
+            self.assertTrue(result)
+        finally:
+            if os.path.isfile(name):
+                os.remove(name)
+
+        # Nested directory and file get's touched
+        name = os.path.join(integration.TMP, 'issue_1864', fname)
+        try:
+            ret = self.run_state(
+                'file.append', name=name, text='cheese', makedirs=True
+            )
+            result = self.state_result(ret)
+            self.assertTrue(result)
+        finally:
+            shutil.rmtree(
+                os.path.join(integration.TMP, 'issue_1864'),
+                ignore_errors=True
+            )
 
     def test_touch(self):
         '''
         file.touch
         '''
         name = os.path.join(integration.TMP, 'touch_test')
-        ret = self.run_state(
-                'file.touch',
-                name=name,
-                )
+        ret = self.run_state('file.touch', name=name)
         self.assertTrue(os.path.isfile(name))
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertTrue(result)
+        os.remove(name)
 
     def test_test_touch(self):
         '''
         file.touch test interface
         '''
         name = os.path.join(integration.TMP, 'touch_test')
-        ret = self.run_state(
-                'file.touch',
-                test=True,
-                name=name,
-                )
+        ret = self.run_state('file.touch', test=True, name=name)
         self.assertFalse(os.path.isfile(name))
-        result = ret[next(iter(ret))]['result']
+        result = self.state_result(ret)
         self.assertIsNone(result)
 
+    def test_touch_directory(self):
+        '''
+        file.touch a directory
+        '''
+        name = os.path.join(integration.TMP, 'touch_test_dir')
+        try:
+            if not os.path.isdir(name):
+                # left behind... Don't fail because of this!
+                os.makedirs(name)
+        except OSError:
+            self.skipTest("Failed to create directory {0}".format(name))
+
+        self.assertTrue(os.path.isdir(name))
+        ret = self.run_state('file.touch', name=name)
+        result = self.state_result(ret)
+        self.assertTrue(result)
+        self.assertTrue(os.path.isdir(name))
+        os.removedirs(name)
+
 
 if __name__ == '__main__':
     from integration import run_tests
